/*
 * Copyright (C) 2022 Ortega Froysa, Nicolás <nicolas@ortegas.org> All rights reserved.
 * Author: Ortega Froysa, Nicolás <nicolas@ortegas.org>
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software
 *    in a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source
 *    distribution.
 */

extern crate rand;

fn main() {
	// set number of prisoners in experiment
	let size = 100;

	// initialize the boxes with capacity for `size` number of boxes
	let mut boxes:Vec<u32> = Vec::with_capacity(size as usize);

	{
		// create vector of numbers to select randomly from
		let mut nums:Vec<u32> = (0..size).collect();

		while nums.len() > 0
		{
			// get a random index for nums
			let rand_i:usize = rand::random::<usize>() % nums.len();
			// add that element to the boxes vector
			boxes.push(nums[rand_i]);
			// remove the entry from the nums vector
			nums.remove(rand_i);
		}
	}

	// assume that all the prisoners have found their number
	let mut all_found = true;
	// for every prisoner in the jail
	for i in 0..size
	{
		// the next (first) box the prisoner is to open is the one with his number (`i`)
		let mut next_box:usize = i as usize;
		// assume the prisoner does not find his box
		let mut found = false;
		// for every attempt the prisoner has (i.e. `size/2`)
		for _j in 0..(size/2)
		{
			// if the number inside the box is his number
			if boxes[next_box] == i
			{
				// assign `found` to true
				found = true;
				// stop looking through more boxes.
				break;
			}
			else
			{
				// if not found set the next_box to the number contained in the current box
				next_box = boxes[next_box] as usize;
			}
		}
		// if a prisoner has not found his number in `size/2` attempts
		if !found
		{
			all_found = false;
		}
	}

	// print result
	if all_found
	{
		println!("All prisoners found their numbers!");
	}
	else
	{
		println!("Some prisoners didn't find their numbers!");
	}
}
